###
### Statistical Analysis and Reporting in R
### Jacob O. Wobbrock, Ph.D.
### The Information School
### University of Washington
### March 12, 2019
### Updated: 03/11/2024
###

###
### Assumptions.R
### (Tests of Assumptions: Normality, Homoscedasticity, Sphericity)
###


###
### Normality
### (Shapiro-Wilk, Anderson-Darling tests)
###

## Shapiro-Wilk normality tests
# (on the response within each condition)
# df has two factors (X1,X2) each w/two levels (a,b) and continuous response (Y)
df <- read.csv("data/Assumptions/2F2LBs_normal.csv")
shapiro.test(df[df$X1 == "a" & df$X2 == "a",]$Y) # condition a,a
shapiro.test(df[df$X1 == "a" & df$X2 == "b",]$Y) # condition a,b
shapiro.test(df[df$X1 == "b" & df$X2 == "a",]$Y) # condition b,a
shapiro.test(df[df$X1 == "b" & df$X2 == "b",]$Y) # condition b,b

## Anderson-Darling normality tests
# (on the response within each condition)
# df has two factors (X1,X2) each w/two levels (a,b) and continuous response (Y)
library(nortest)
df <- read.csv("data/Assumptions/2F2LBs_normal.csv")
ad.test(df[df$X1 == "a" & df$X2 == "a",]$Y) # condition a,a
ad.test(df[df$X1 == "a" & df$X2 == "b",]$Y) # condition a,b
ad.test(df[df$X1 == "b" & df$X2 == "a",]$Y) # condition b,a
ad.test(df[df$X1 == "b" & df$X2 == "b",]$Y) # condition b,b


## Shapiro-Wilk, Anderson-Darling normality tests
# (on between-Ss residuals)
# df has two between-Ss factors (X1,X2) each w/two levels (a,b) and continuous response (Y)
library(nortest)
df <- read.csv("data/Assumptions/2F2LBs_normal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"

m = aov(Y ~ X1*X2, data=df) # make anova model
r = residuals(m)            # extract model residuals
mean(r); sum(r)             # both should be ~0
par(mfrow=c(3,1))
  plot(r[1:length(r)], main="Residual plot"); abline(h=0) # should look random
  hist(r, main="Histogram of residuals") # should look normal
  qqnorm(r); qqline(r) # Q-Q plot
par(mfrow=c(1,1))
shapiro.test(r) # Shapiro-Wilk test
ad.test(r)      # Anderson-Darling test


## Shapiro-Wilk, Anderson-Darling normality tests
# (on within-Ss residuals)
# df has two within-Ss factors (X1,X2) each w/two levels (a,b) and continuous response (Y)
library(afex)
library(nortest)
df <- read.csv("data/Assumptions/2F2LWs_normal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"

m = aov_ez(dv="Y", within=c("X1","X2"), id="S", type=3, data=df) # make rm-anova model
r = residuals(m$lm)  # extract model residuals
mean(r); sum(r)      # both should be ~0
par(mfrow=c(3,1))
  plot(r[1:length(r)], main="Residual plot"); abline(h=0) # should look random
  hist(r, main="Histogram of residuals") # should look normal
  qqnorm(r); qqline(r) # Q-Q plot
par(mfrow=c(1,1))
shapiro.test(r) # Shapiro-Wilk test
ad.test(r)      # Anderson-Darling test


## Shapiro-Wilk, Anderson-Darling normality tests
# (on residuals from linear mixed models)
# df has two factors (X1,X2) each w/two levels (a,b) and continuous response (Y)
library(lme4)
library(lmerTest)
library(nortest)
df <- read.csv("data/Assumptions/2F2LWs_normal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"

m = lmer(Y ~ X1*X2 + (1|S), data=df) # make linear mixed model
r = residuals(m)                     # extract model residuals
mean(r); sum(r)                      # both should be ~0
par(mfrow=c(3,1))
  plot(r[1:length(r)], main="Residual plot"); abline(h=0) # should look random
  hist(r, main="Histogram of residuals") # should look normal
  qqnorm(r); qqline(r) # Q-Q plot
par(mfrow=c(1,1))
shapiro.test(r) # Shapiro-Wilk test
ad.test(r)      # Anderson-Darling test



###
### Homoscedasticity
### (Levene's test)
###

# df has subjects (S), one between-Ss factor (X1), one within-Ss factor (X2),
# and continuous response (Y)
library(afex)        # for aov_ez
library(performance) # for check_homogeneity
df <- read.csv("data/Assumptions/2F2LMs_normal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"

m = aov_ez(dv="Y", between="X1", within="X2", id="S", type=3, data=df)
print(check_homogeneity(m)) # Levene's test

# if a violation occurs (p<.05), use a Welch t-test for one factor...
t.test(Y ~ X1, data=df, var.equal=FALSE)

# ...or a White-adjusted ANOVA for >1 factor
library(car) # for Anova
Anova(m$lm, type=3, white.adjust=TRUE) # shows between-Ss factors only



###
### Sphericity
### (Mauchly's test)
###

# df has subjects (S), one between-Ss factor (X1), one within-Ss factor (X2),
# and continuous response (Y)
library(afex) # for aov_ez
df <- read.csv("data/Assumptions/2F23LMs_mauchly.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"

m = aov_ez(dv="Y", between="X1", within="X2", id="S", type=3, data=df)
s = summary(m)
s$sphericity.tests  # Mauchly's sphericity tests
s$univariate.tests  # if p>.05, use these unadjusted DFs and p-values
anova(m)            # if p<.05, use these G-G adjusted DFs and p-values


