###
### Statistical Analysis and Reporting in R
### Jacob O. Wobbrock, Ph.D.
### The Information School
### University of Washington
### March 12, 2019
### Updated: 03/12/2024
###

###
### GLM.R
### (Generalized Linear Models)
###


##
## Normal
##
library(car) # for Anova
df <- read.csv("data/GLM/1F3LBs_normal.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glm(Y ~ X, data=df, family=gaussian)
Anova(m, type=3, test.statistic="F")

## Normal post hoc test
# df has subjects (S), one between-Ss factor (X) w/levels (a,b,c), and continuous response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Binomial
##
library(car) # for Anova
df <- read.csv("data/GLM/1F3LBs_binomial.csv")
df$S = factor(df$S)
df$X = factor(df$X)
df$Y = factor(df$Y) # nominal response
contrasts(df$X) <- "contr.sum"
m = glm(Y ~ X, data=df, family=binomial)
Anova(m, type=3)

## Binomial post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and dichotomous response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Multinomial
##
library(nnet) # for multinom
library(car)  # for Anova
df <- read.csv("data/GLM/1F3LBs_multinomial.csv")
df$S = factor(df$S)
df$X = factor(df$X)
df$Y = factor(df$Y) # nominal response
contrasts(df$X) <- "contr.sum"
m = multinom(Y ~ X, data=df)
Anova(m, type=3)

## Multinomial post hoc tests
# df has subjects (S), one between-Ss factor (X) w/levels (a,b,c), and polytomous response (Y)
library(nnet) # for multinom
library(car)  # for Anova

df2 <- df[df$X == "a" | df$X == "b",] # a vs. b
df2$X = factor(df2$X)
contrasts(df2$X) <- "contr.sum"
m = multinom(Y ~ X, data=df2) 
ab = Anova(m, type=3)

df2 <- df[df$X == "a" | df$X == "c",] # a vs. c
df2$X = factor(df2$X)
contrasts(df2$X) <- "contr.sum"
m = multinom(Y ~ X, data=df2) 
ac = Anova(m, type=3)

df2 <- df[df$X == "b" | df$X == "c",] # b vs. c
df2$X = factor(df2$X)
contrasts(df2$X) <- "contr.sum"
m = multinom(Y ~ X, data=df2) 
bc = Anova(m, type=3)

p.adjust(c(ab$`Pr(>Chisq)`, ac$`Pr(>Chisq)`, bc$`Pr(>Chisq)`), method="holm")



##
## Ordinal
##
library(MASS) # for polr
library(car)  # for Anova
df <- read.csv("data/GLM/1F3LBs_ordinal.csv")
df$S = factor(df$S)
df$X = factor(df$X)
df$Y = ordered(df$Y) # ordinal response
contrasts(df$X) <- "contr.sum"
m = polr(Y ~ X, data=df, Hess=TRUE)
Anova(m, type=3)

## Ordinal post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and ordinal response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans) # for as.glht, pairs, emmeans
summary(as.glht(pairs(emmeans(m, ~ X))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Poisson
##
library(car) # for Anova
df <- read.csv("data/GLM/1F3LBs_poisson.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glm(Y ~ X, data=df, family=poisson)
Anova(m, type=3)

## Poisson post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans) # emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Zero-Inflated Poisson
##
library(pscl)        # for zeroinfl
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLM/1F3LBs_zipoisson.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = zeroinfl(Y ~ X, data=df, dist="poisson")
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Poisson post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for adjusted
library(emmeans) # for as.glht, pairs, emmeans
summary(as.glht(pairs(emmeans(m, ~ X))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Negative Binomial
##
library(MASS)        # for glm.nb
library(car)         # for Anova
df <- read.csv("data/GLM/1F3LBs_negbin.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glm.nb(Y ~ X, data=df)
Anova(m, type=3)

## Negative Binomial post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Zero-Inflated Negative Binomial
##
library(pscl)        # for zeroinfl
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLM/1F3LBs_zinegbin.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = zeroinfl(Y ~ X, data=df, dist="negbin")
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Poisson post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for adjusted
library(emmeans)  # for as.glht, pairs, emmeans
summary(as.glht(pairs(emmeans(m, ~ X))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Gamma, including Exponential
##
library(car) # for Anova
df <- read.csv("data/GLM/1F3LBs_gamma.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glm(Y ~ X, data=df, family=Gamma) # family=Gamma(link="log") is often used
Anova(m, type=3)

## Gamma post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and continuous response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs similar contrast tests
library(emmeans) # emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")




###
### Multiple Between-Ss Factors
###


##
## Normal
##
library(car) # for Anova
df <- read.csv("data/GLM/2F2LBs_normal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glm(Y ~ X1*X2, data=df, family=gaussian)
Anova(m, type=3, test.statistic="F")

## Normal post hoc test
# df has subjects (S), two between-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(0,20),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LBs_normal.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Binomial
##
library(car) # for Anova
df <- read.csv("data/GLM/2F2LBs_binomial.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
df$Y = factor(df$Y) # nominal response
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glm(Y ~ X1*X2, data=df, family=binomial)
Anova(m, type=3)

## Binomial post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and dichotomous response (Y)
library(multcomp) # for glht
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create a barplot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df2 <- as.data.frame(xtabs(~ X1 + X2 + Y, data=df))
df2$X12 = with(df2, interaction(X1, X2))
df2$X12 = factor(df2$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- ggbarplot(df2, x="X12", y="Freq", fill="Y", 
               palette=c("#69b3a2","#404080"),
               alpha=0.25,
               title="Y by X1, X2",
               subtitle="Barplots",
               label=TRUE,
               lab.pos="in",
               lab.col="black",
               xlab="X1.X2",
               ylab="Count",
               caption="2F2LBs_binomial.csv",
               legend="right",
               position=position_dodge(0.75),
               ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Multinomial
##
library(nnet) # for multinom
library(car)  # for Anova
df <- read.csv("data/GLM/2F2LBs_multinomial.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
df$Y = factor(df$Y) # nominal response
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = multinom(Y ~ X1*X2, data=df)
Anova(m, type=3)

## Multinomial post hoc tests
# df has subjects (S), two between-Ss factors (X1,X2) each w/levels (a,b), and polytomous response (Y)
library(nnet) # for multinom
library(car)  # for Anova
df$X12 = with(df, interaction(X1,X2))

df2 <- df[df$X12 == "a.a" | df$X12 == "a.b",] # aa vs. ab
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_ab = Anova(m, type=3)

df2 <- df[df$X12 == "a.a" | df$X12 == "b.a",] # aa vs. ba
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_ba = Anova(m, type=3)

df2 <- df[df$X12 == "a.a" | df$X12 == "b.b",] # aa vs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_bb = Anova(m, type=3)

df2 <- df[df$X12 == "a.b" | df$X12 == "b.a",] # ab vs. ba
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ab_ba = Anova(m, type=3)

df2 <- df[df$X12 == "a.b" | df$X12 == "b.b",] # ab vs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ab_bb = Anova(m, type=3)

df2 <- df[df$X12 == "b.a" | df$X12 == "b.b",] # ba vs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ba_bb = Anova(m, type=3)

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), method="holm")

# Create a barplot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df2 <- as.data.frame(xtabs(~ X1 + X2 + Y, data=df))
df2$X12 = with(df2, interaction(X1, X2))
df2$X12 = factor(df2$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- ggbarplot(df2, x="X12", y="Freq", fill="Y", 
               palette=c("#69b3a2","#404080","#e69f00"),
               alpha=0.25,
               title="Y by X1, X2",
               subtitle="Barplots",
               label=TRUE,
               lab.pos="in",
               lab.col="black",
               xlab="X1.X2",
               ylab="Count",
               ylim=c(-1,15),
               caption="2F2LBs_multinomial.csv",
               legend="right",
               position=position_dodge(0.75),
               ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Ordinal
##
library(MASS) # for polr
library(car)  # for Anova
df <- read.csv("data/GLM/2F2LBs_ordinal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
df$Y = ordered(df$Y) # ordinal response
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = polr(Y ~ X1*X2, data=df, Hess=TRUE)
Anova(m, type=3)

## Ordinal post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and ordinal response (Y)
library(multcomp) # for adjusted
library(emmeans)  # for as.glht, pairs, emmeans
summary(as.glht(pairs(emmeans(m, ~ X1*X2))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create barplots as histograms
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df2 <- as.data.frame(xtabs(~ X1 + X2 + Y, data=df))
df2$X12 = with(df2, interaction(X1, X2))
df2$X12 = factor(df2$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- ggbarplot(df2, x="Y", y="Freq", fill="X12",
               facet.by="X12",
               panel.labs.font=list(size=14, face="bold"),
               palette=c("#69b3a2","#404080","#e69f00","#aa0000"),
               alpha=0.25,
               title="Y by X1, X2",
               subtitle="Barplots",
               label=TRUE,
               lab.pos="in",
               lab.col="black",
               xlab="X1.X2",
               ylab="Count",
               ylim=c(-1,6),
               caption="2F2LBs_ordinal.csv",
               legend="right",
               legend.title="X1.X2",
               ggtheme=theme_minimal()
)
p + 
  facet_grid(X2 ~ X1) + 
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Poisson
##
library(car) # for Anova
df <- read.csv("data/GLM/2F2LBs_poisson.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glm(Y ~ X1*X2, data=df, family=poisson)
Anova(m, type=3)

## Poisson post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for glht
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-2,13),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LBs_poisson.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Zero-Inflated Poisson
##
library(pscl)        # for zeroinfl
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLM/2F2LBs_zipoisson.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = zeroinfl(Y ~ X1*X2, data=df, dist="poisson")
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Poisson post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for adjusted
library(emmeans) # for as.glht, pairs, emmeans
summary(as.glht(pairs(emmeans(m, ~ X1*X2))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create histograms
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df$X12 = with(df, interaction(X1, X2))
df$X12 = factor(df$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- gghistogram(df, x="Y", fill="X12",
                 facet.by="X12",
                 panel.labs.font=list(size=14, face="bold"),
                 palette=c("#69b3a2","#404080","#e69f00","#aa0000"),
                 alpha=0.25,
                 bins=10,
                 rug=TRUE,
                 add_density=TRUE,
                 xlim=c(-0.25,10.25),
                 xticks.by=1,
                 ylim=c(0,7),
                 ylab="Count",
                 title="Y by X1, X2",
                 subtitle="Histograms",
                 caption="2F2LBs_zipoisson.csv",
                 legend="right",
                 legend.title="X1.X2",
                 ggtheme=theme_minimal()
)
p + 
  facet_grid(X12 ~ .) +  # stacked
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14) + 
  geom_text(stat="bin", aes(y=after_stat(density), label=stat(count)), bins=10, vjust=-0.2)



##
## Negative Binomial
##
library(MASS)        # for glm.nb
library(car)         # for Anova
df <- read.csv("data/GLM/2F2LBs_negbin.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glm.nb(Y ~ X1*X2, data=df)
Anova(m, type=3)

## Negative Binomial post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-5,20),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LBs_negbin.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Zero-Inflated Negative Binomial
##
library(pscl)        # for zeroinfl
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLM/2F2LBs_zinegbin.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = zeroinfl(Y ~ X1*X2, data=df, dist="negbin")
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Poisson post hoc test
# df has subjects (S), two between-Ss factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for adjusted
library(emmeans) # for as.glht, pairs, emmeans
summary(as.glht(pairs(emmeans(m, ~ X1*X2))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create histograms
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df$X12 = with(df, interaction(X1, X2))
df$X12 = factor(df$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- gghistogram(df, x="Y", fill="X12",
                 facet.by="X12",
                 panel.labs.font=list(size=14, face="bold"),
                 palette=c("#69b3a2","#404080","#e69f00","#aa0000"),
                 alpha=0.25,
                 bins=10,
                 rug=TRUE,
                 add_density=TRUE,
                 xlim=c(-0.25,13.25),
                 xticks.by=1,
                 ylim=c(0,10),
                 yticks.by=2,
                 ylab="Count",
                 title="Y by X1, X2",
                 subtitle="Histograms",
                 caption="2F2LBs_zinegbin.csv",
                 legend="right",
                 legend.title="X1.X2",
                 ggtheme=theme_minimal()
)
p + 
  facet_grid(X12 ~ .) +  # stacked
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14) + 
  geom_text(stat="bin", aes(y=after_stat(density), label=stat(count)), bins=10, vjust=-0.2)



##
## Gamma, including Exponential
##
library(car) # for Anova
df <- read.csv("data/GLM/2F2LBs_gamma.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glm(Y ~ X1*X2, data=df, family=Gamma) # family=Gamma(link="log") is an alternative
Anova(m, type=3)

## Gamma post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-3,17),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LBs_gamma.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)


