###
### Statistical Analysis and Reporting in R
### Jacob O. Wobbrock, Ph.D.
### The Information School
### University of Washington
### March 12, 2019
### Updated: 03/12/2024
###

###
### GLMM.R
### (Generalized Linear Mixed Models)
###


##
## Normal
##
library(lme4) # for glmer, lmer
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/1F3LWs_normal.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
#m = glmer(Y ~ X + (1|S), data=df, family=gaussian) #deprecated
m = lmer(Y ~ X + (1|S), data=df) # equivalent, use instead
Anova(m, type=3, test.statistic="F")

## Normal post hoc test
# df has subjects (S), one within-Ss factor (X) w/levels (a,b,c), and continuous response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Binomial
##
library(lme4) # for glmer
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/1F3LWs_binomial.csv")
df$S = factor(df$S)
df$X = factor(df$X)
df$Y = factor(df$Y) # nominal response
contrasts(df$X) <- "contr.sum"
m = glmer(Y ~ X + (1|S), data=df, family=binomial)
Anova(m, type=3)

## Binomial post hoc test
# df has subjects (S), one within-Ss factor (X) w/levels (a,b,c), and dichotomous response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Multinomial
##
library(mlogit) # for mlogit.data
library(lme4)   # for glmer
library(lmerTest)
library(car)    # for Anova
df <- read.csv("data/GLMM/1F3LWs_multinomial.csv")
df$S = factor(df$S)
df$X = factor(df$X)
df$Y = factor(df$Y) # nominal response

# use the multinomial-Poisson (m-P) transformation (Baker 1994)
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X)   <- "contr.sum"
contrasts(df2$alt) <- "contr.sum"

m = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson) # m-P
a = Anova(m, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

## Multinomial post hoc test
# df has subjects (S), one within-Ss factor (X) w/levels (a,b,c), and polytomous response (Y)
# df2 was produced by mlogit.data and has a logical response (Y) and alt factor
library(lme4) # for glmer
library(lmerTest)
library(car) # for Anova

df3 <- df2[df2$X == "a" | df2$X == "b",] # a vs. b
df3$X = factor(df3$X)
contrasts(df3$X) <- "contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab = a[grep(":alt", rownames(a)),] 

df3 <- df2[df2$X == "a" | df2$X == "c",] # a vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <- "contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson) 
a = Anova(m, type=3)
ac = a[grep(":alt", rownames(a)),] 

df3 <- df2[df2$X == "b" | df2$X == "c",] # b vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <- "contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)  
a = Anova(m, type=3)
bc = a[grep(":alt", rownames(a)),]

p.adjust(c(ab$`Pr(>Chisq)`, ac$`Pr(>Chisq)`, bc$`Pr(>Chisq)`), method="holm")



##
## Ordinal
##
library(ordinal)         # for clmm
library(RVAideMemoire)   # for Anova.clmm
df <- read.csv("data/GLMM/1F3LWs_ordinal.csv")
df$S = factor(df$S)
df$X = factor(df$X)
df$Y = ordered(df$Y)     # ordinal response
contrasts(df$X) <- "contr.sum"
df2 <- as.data.frame(df) # Anova.clmm fails without this
m = clmm(Y ~ X + (1|S), data=df2, link="logit") # or "probit"
Anova.clmm(m)

## Ordinal post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and ordinal response (Y)
library(multcomp) # for adjusted
library(emmeans)  # for as.glht, pairs, emmeans
summary(as.glht(pairs(emmeans(m, ~ X))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Poisson
##
library(lme4) # for glmer
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/1F3LWs_poisson.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glmer(Y ~ X + (1|S), data=df, family=poisson)
Anova(m, type=3)

## Poisson post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Zero-Inflated Poisson
##
library(glmmTMB)     # for glmmTMB
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLMM/1F3LWs_zipoisson.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glmmTMB(Y ~ X + (1|S), data=df, family=poisson, ziformula=~X)
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Poisson post hoc test
# df has subjects (S), one within-Ss factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Negative Binomial
##
library(lme4) # for glmer.nb
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/1F3LWs_negbin.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glmer.nb(Y ~ X + (1|S), data=df)
Anova(m, type=3)

## Negative Binomial post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Zero-Inflated Negative Binomial
##
library(glmmTMB)     # for glmmTMB
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLMM/1F3LWs_zinegbin.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glmmTMB(Y ~ X + (1|S), data=df, family=nbinom2, ziformula=~X)
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Poisson post hoc test
# df has subjects (S), one within-Ss factor (X) w/levels (a,b,c), and count response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## Gamma, including Exponential
##
library(lme4) # for glmer
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/1F3LWs_gamma.csv")
df$S = factor(df$S)
df$X = factor(df$X)
contrasts(df$X) <- "contr.sum"
m = glmer(Y ~ X + (1|S), data=df, family=Gamma)
# family=Gamma(link="log") is often used
Anova(m, type=3)

## Gamma post hoc test
# df has subjects (S), one factor (X) w/levels (a,b,c), and continuous response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm"))
# the following also performs the equivalent contrast tests
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")




###
### Multiple Within-Ss Factors
###


##
## Normal
##
library(lme4) # for glmer, lmer
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/2F2LWs_normal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
#m = glmer(Y ~ X1*X2 + (1|S), data=df, family=gaussian) #deprecated
m = lmer(Y ~ X1*X2 + (1|S), data=df) # equivalent, use instead
Anova(m, type=3, test.statistic="F")

## Normal post hoc test
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(multcomp) # for glht
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-10,30),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LWs_normal.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Binomial
##
library(lme4) # for glmer
library(lmerTest)
library(car) # for Anova
df <- read.csv("data/GLMM/2F2LWs_binomial.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
df$Y = factor(df$Y) # nominal response
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glmer(Y ~ X1*X2 + (1|S), data=df, family=binomial)
Anova(m, type=3)

## Binomial post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and dichotomous response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create a barplot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df2 <- as.data.frame(xtabs(~ X1 + X2 + Y, data=df))
df2$X12 = with(df2, interaction(X1, X2))
df2$X12 = factor(df2$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- ggbarplot(df2, x="X12", y="Freq", fill="Y", 
               palette=c("#69b3a2","#404080"),
               alpha=0.25,
               title="Y by X1, X2",
               subtitle="Barplots",
               label=TRUE,
               lab.pos="in",
               lab.col="black",
               xlab="X1.X2",
               ylab="Count",
               caption="2F2LWs_binomial.csv",
               legend="right",
               position=position_dodge(0.75),
               ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Multinomial
##
library(mlogit) # for mlogit.data
library(lme4) # for glmer
library(lmerTest)
library(car) # for Anova
df <- read.csv("data/GLMM/2F2LWs_multinomial.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
df$Y = factor(df$Y) # nominal response

# use the multinomial-Poisson (m-P) transformation (Baker 1994)
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X1) <- "contr.sum"
contrasts(df2$X2) <- "contr.sum"
contrasts(df2$alt) <- "contr.sum"

m = glmer(Y ~ X1*X2*alt + (1|S), data=df2, family=poisson) # m-P
a = Anova(m, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

## Multinomial post hoc test
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and polytomous response (Y)
# df2 was produced by mlogit.data and has a logical response (Y) and alt factor
library(lme4) # for glmer
library(lmerTest)
library(car) # for Anova
df2$X12 = with(df2, interaction(X1,X2)) # make combined factor for contrasts

df3 <- df2[df2$X12 == "a.a" | df2$X12 == "a.b",] # aa vs. ab
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <- "contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ab = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X12 == "a.a" | df2$X12 == "b.a",] # aa vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <- "contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ba = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X12 == "a.a" | df2$X12 == "b.b",] # aa vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <- "contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_bb = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X12 == "a.b" | df2$X12 == "b.a",] # ab vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <- "contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_ba = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X12 == "a.b" | df2$X12 == "b.b",] # ab vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <- "contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_bb = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X12 == "b.a" | df2$X12 == "b.b",] # ba vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <- "contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ba_bb = a[grep(":alt", rownames(a)),]

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), method="holm")

# Create a barplot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df2 <- as.data.frame(xtabs(~ X1 + X2 + Y, data=df))
df2$X12 = with(df2, interaction(X1, X2))
df2$X12 = factor(df2$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- ggbarplot(df2, x="X12", y="Freq", fill="Y", 
               palette=c("#69b3a2","#404080","#e69f00"),
               alpha=0.25,
               title="Y by X1, X2",
               subtitle="Barplots",
               label=TRUE,
               lab.pos="in",
               lab.col="black",
               xlab="X1.X2",
               ylab="Count",
               ylim=c(0,10),
               caption="2F2LWs_multinomial.csv",
               legend="right",
               position=position_dodge(0.75),
               ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Ordinal
##
library(ordinal)         # for clmm
library(RVAideMemoire)   # for Anova.clmm
df <- read.csv("data/GLMM/2F2LWs_ordinal.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
df$Y = ordered(df$Y)     # ordinal response
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
df2 <- as.data.frame(df) # Anova.clmm fails without this
m = clmm(Y ~ X1*X2 + (1|S), data=df2, link="logit") # or "probit"
Anova.clmm(m)

## Ordinal post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and ordinal response (Y)
library(multcomp) # for adjusted
library(emmeans)  # for as.glht, pairs
summary(as.glht(pairs(emmeans(m, ~ X1*X2))), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create barplots as histograms
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df2 <- as.data.frame(xtabs(~ X1 + X2 + Y, data=df))
df2$X12 = with(df2, interaction(X1, X2))
df2$X12 = factor(df2$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- ggbarplot(df2, x="Y", y="Freq", fill="X12",
               facet.by="X12",
               palette=c("#69b3a2","#404080","#e69f00","#aa0000"),
               alpha=0.25,
               title="Y by X1, X2",
               subtitle="Barplots",
               label=TRUE,
               lab.pos="in",
               lab.col="black",
               xlab="X1.X2",
               ylab="Count",
               ylim=c(-1,6),
               caption="2F2LWs_ordinal.csv",
               legend="right",
               legend.title="X1.X2",
               panel.labs.font=list(size=14, face="bold"),
               ggtheme=theme_minimal()
)
p + 
  facet_grid(X2 ~ X1) + 
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Poisson
##
library(lme4) # for glmer
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/2F2LWs_poisson.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glmer(Y ~ X1*X2 + (1|S), data=df, family=poisson)
Anova(m, type=3)

## Poisson post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-1,16),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LWs_poisson.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Zero-Inflated Poisson
##
library(glmmTMB)     # for glmmTMB
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLMM/2F2LWs_zipoisson.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glmmTMB(Y ~ X1*X2 + (1|S), data=df, family=poisson, ziformula=~X1*X2)
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Poisson post hoc test
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create histograms
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df$X12 = with(df, interaction(X1, X2))
df$X12 = factor(df$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- gghistogram(df, x="Y", fill="X12",
                 facet.by="X12",
                 panel.labs.font=list(size=14, face="bold"),
                 palette=c("#69b3a2","#404080","#e69f00","#aa0000"),
                 alpha=0.25,
                 bins=10,
                 rug=TRUE,
                 add_density=TRUE,
                 xlim=c(-0.25,10.25),
                 xticks.by=1,
                 ylim=c(0,9),
                 ylab="Count",
                 title="Y by X1, X2",
                 subtitle="Histograms",
                 caption="2F2LWs_zipoisson.csv",
                 legend="right",
                 legend.title="X1.X2",
                 ggtheme=theme_minimal()
)
p + 
  facet_grid(X12 ~ .) +  # stacked
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14) + 
  geom_text(stat="bin", aes(y=after_stat(density), label=stat(count)), bins=10, vjust=-0.2)



##
## Negative Binomial
##
library(lme4) # for glmer.nb
library(lmerTest)
library(car)  # for Anova
df <- read.csv("data/GLMM/2F2LWs_negbin.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glmer.nb(Y ~ X1*X2 + (1|S), data=df)
Anova(m, type=3)

## Negative Binomial post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for glht
library(emmeans)  # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-10,55),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LWs_negbin.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



##
## Zero-Inflated Negative Binomial
##
library(glmmTMB)     # for glmmTMB
library(car)         # for Anova
library(performance) # for check_zeroinflation
df <- read.csv("data/GLMM/2F2LWs_zinegbin.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glmmTMB(Y ~ X1*X2 + (1|S), data=df, family=nbinom2, ziformula=~X1*X2)
check_zeroinflation(m)
Anova(m, type=3)

## Zero-Inflated Negative Binomial post hoc test
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and count response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create histograms
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
df$X12 = with(df, interaction(X1, X2))
df$X12 = factor(df$X12, levels=c("a.a","a.b","b.a","b.b"))
p <- gghistogram(df, x="Y", fill="X12",
                 facet.by="X12",
                 panel.labs.font=list(size=14, face="bold"),
                 palette=c("#69b3a2","#404080","#e69f00","#aa0000"),
                 alpha=0.25,
                 bins=10,
                 rug=TRUE,
                 add_density=TRUE,
                 xlim=c(-0.25,25),
                 xticks.by=1,
                 ylim=c(0,8),
                 yticks.by=2,
                 ylab="Count",
                 title="Y by X1, X2",
                 subtitle="Histograms",
                 caption="2F2LWs_zinegbin.csv",
                 legend="right",
                 legend.title="X1.X2",
                 ggtheme=theme_minimal()
)
p + 
  facet_grid(X12 ~ .) +  # stacked
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14) + 
  geom_text(stat="bin", aes(y=after_stat(density), label=stat(count)), bins=10, vjust=-0.2)



##
## Gamma, including Exponential
##
library(lme4) # for glmer
library(lmerTest)
library(car) # for Anova
df <- read.csv("data/GLMM/2F2LWs_gamma.csv")
df$S = factor(df$S)
df$X1 = factor(df$X1)
df$X2 = factor(df$X2)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = glmer(Y ~ X1*X2 + (1|S), data=df, family=Gamma) # family=Gamma(link="log") is an alternative
Anova(m, type=3)

## Gamma post hoc test
# df has subjects (S), two factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-1.5,16),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LWs_gamma.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)


