###
### Statistical Analysis and Reporting in R
### Jacob O. Wobbrock, Ph.D.
### The Information School
### University of Washington
### November 13, 2019
### Updated: 03/12/2024
###

###
### Parametric.R
### (Parametric Analyses of Differences)
###


###
### One Factor
###

## Independent-samples t-test
# df has subjects (S), one between-Ss factor (X) w/levels (a,b), and continuous response (Y)
df <- read.csv("data/Parametric/1F2LBs.csv")
df$S = factor(df$S) # Subject id is nominal (unused)
df$X = factor(df$X) # X is a 2-level factor
t.test(Y ~ X, data=df, var.equal=TRUE) # use var.equal=FALSE if heteroscedastistic



##
## Paired-samples t-test
##
# df has subjects (S), one within-Ss factor (X) w/levels (a,b), and continuous response (Y)
library(reshape2) # for dcast
df <- read.csv("data/Parametric/1F2LWs.csv")
df$S = factor(df$S) # Subject id is nominal
df$X = factor(df$X) # X is a 2-level factor
df2 <- dcast(df, S ~ X, value.var="Y") # make wide-format table
t.test(df2$a, df2$b, paired=TRUE) # homoscedasticity is irrelevant for a paired-samples t-test



##
## One-way ANOVA
##
# df has subjects (S), one between-Ss factor (X) w/levels (a,b,c), and continuous response (Y)
df <- read.csv("data/Parametric/1F3LBs.csv")
df$S = factor(df$S)     # Subject id is nominal (unused)
df$X = factor(df$X)     # X is a 3-level factor
m = aov(Y ~ X, data=df) # fit model
anova(m)

## One-way ANOVA post hoc test
# df has subjects (S), one between-Ss factor (X) w/levels (a,b,c), and continuous response (Y)
library(multcomp) # for glht, mcp
summary(glht(m, mcp(X="Tukey")), test=adjusted(type="holm")) # m is from aov
# the following also performs the same contrast tests
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X)), test=adjusted(type="holm"))
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")



##
## One-way repeated measures ANOVA
##
# df has subjects (S), one within-Ss factor (X) w/levels (a,b,c), and continuous response (Y)
library(afex)       # for aov_ez
df <- read.csv("data/Parametric/1F3LWs.csv")
df$S = factor(df$S) # Subject id is nominal
df$X = factor(df$X) # X is a 3-level factor
m = aov_ez(dv="Y", within="X", id="S", type=3, data=df)
s = summary(m)
s$sphericity.tests  # Mauchly's sphericity tests
s$univariate.tests  # if p>.05, use these unadjusted DFs and p-values
anova(m)            # if p<.05, use these G-G adjusted DFs and p-values

# the following also performs the equivalent repeated measures ANOVA, but does not address sphericity
m0 = aov(Y ~ X + Error(S/X), data=df)
summary(m0)

## One-way repeated measures ANOVA post hoc test
# df has subjects (S), one within-Ss factor (X) w/levels (a,b,c), and continuous response (Y)
library(emmeans) # for emmeans
emmeans(m, pairwise ~ X, adjust="holm") # m is from afex::aov_ez
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X, adjust="tukey")




###
### Multiple Between-Ss Factors
###

##
## Factorial ANOVA
##
# df has subjects (S), two between-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(afex)         # for aov_ez
library(performance)  # for check_homogeneity
library(car)          # for Anova
df <- read.csv("data/Parametric/2F2LBs.csv")
df$S = factor(df$S)   # Subject id is nominal
df$X1 = factor(df$X1) # X1 is a 2-level factor
df$X2 = factor(df$X2) # X2 is a 2-level factor
m = aov_ez(dv="Y", between=c("X1","X2"), id="S", type=3, data=df)
print(check_homogeneity(m))             # Levene's test
anova(m)                                # if p>.05 from Levene's test
Anova(m$lm, type=3, white.adjust=TRUE)  # if p<.05 from Levene's test

# the following also performs the same factorial ANOVA
m0 = aov(Y ~ X1*X2, data=df)
anova(m0)

## Factorial ANOVA post hoc tests
# df has subjects (S), two between-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(emmeans) # for emmeans
emmeans(m, pairwise ~ X1*X2, adjust="holm") # m is from afex::aov_ez
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")



##
## Linear Model (LM)
##
# df has subjects (S), two between-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
df <- read.csv("data/Parametric/2F2LBs.csv")
df$S = factor(df$S)   # Subject id is nominal (unused)
df$X1 = factor(df$X1) # X1 is a 2-level factor
df$X2 = factor(df$X2) # X2 is a 2-level factor
m = lm(Y ~ X1*X2, data=df)
anova(m)

## Linear model post hoc tests
# df has subjects (S), two between-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm")) # m is from aov or lm
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(0,30),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LBs.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)



###
### Multiple Within-Ss Factors
###

##
## Factorial repeated measures ANOVA
##
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(afex) # for aov_ez
df <- read.csv("data/Parametric/2F2LWs.csv")
df$S = factor(df$S)   # Subject id is nominal
df$X1 = factor(df$X1) # X1 is a 2-level factor
df$X2 = factor(df$X2) # X2 is a 2-level factor
m = aov_ez(dv="Y", within=c("X1","X2"), id="S", type=3, data=df)
s = summary(m)
s$sphericity.tests    # Mauchly's sphericity tests
s$univariate.tests    # if p>.05, use these unadjusted DFs and p-values
anova(m)              # if p<.05, use these G-G adjusted DFs and p-values

# the following also performs the equivalent repeated measures ANOVA, but does not address sphericity
m0 = aov(Y ~ X1*X2 + Error(S/(X1*X2)), data=df)
summary(m0)

## Factorial repeated measures ANOVA post hoc test
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(emmeans) # for emmeans
emmeans(m, pairwise ~ X1*X2, adjust="holm") # m is from afex::aov_ez
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")



##
## Linear Mixed Model (LMM)
##
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(lme4)     # for lmer
library(lmerTest)
library(car)      # for Anova
df <- read.csv("data/Parametric/2F2LWs.csv")
df$S = factor(df$S) # Subject id is nominal
df$X1 = factor(df$X1) # X1 is a 2-level factor
df$X2 = factor(df$X2) # X2 is a 2-level factor
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
m = lmer(Y ~ X1*X2 + (1|S), data=df)
Anova(m, type=3, test.statistic="F")

## Linear Mixed Model (LMM) post hoc tests
# df has subjects (S), two within-Ss factors (X1,X2) each w/levels (a,b), and continuous response (Y)
library(multcomp) # for glht
library(emmeans) # for emm, emmeans
summary(glht(m, emm(pairwise ~ X1*X2)), test=adjusted(type="holm")) # m is from lmer
# or, using the Tukey HSD correction instead of Holm's
emmeans(m, pairwise ~ X1*X2, adjust="tukey")

# Create an interaction plot
# http://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/
# https://www.datanovia.com/en/blog/tag/ggpubr/ 
library(ggpubr)
p <- ggline(df, x="X1", y="Y", color="X2", 
            palette=c("#69b3a2","#404080"),
            size=1,
            point.size=2,
            add=c("mean_sd","violin"),
            add.params=list(fill="X2",alpha=0.15),
            position=position_dodge(.05),
            ylim=c(-10,40),
            title="Y by X1, X2",
            subtitle="Violin Interaction Plot",
            caption="2F2LWs.csv",
            legend="right",
            ggtheme=theme_minimal()
)
p +  # set fonts
  font("title", size=24, face="bold") + 
  font("legend.title", size=14, face="bold") + 
  font("legend.text", size=14) + 
  font("xylab", size=14, face="bold") + 
  font("xy.text", size=14)

