###
### Statistical Analysis and Reporting in R
### Jacob O. Wobbrock, Ph.D.
### The Information School
### University of Washington
### February 24, 2024
###

###
### m-P.R
### multinomial-Poisson transformation test file
###


### This file tests the viability of the multinomial-Poisson (m-P)
### transformation (Baker 1994). Fourteen scenarios are explored that
### compare results from glm(family=binomial), nnet::multinom(), 
### lme4::glmer(family=binomial), and the m-P approach. (The m-P approach
### is carried out by glm(family=poisson), when possible, and 
### glmer(family=poisson) in every case.) Results show that when 
### glm(family=poisson) is usable (it can only handle between-Ss. factors), 
### its m-P results always agree with those of nnet::multinom(), confirming
### the effectiveness of the m-P transformation. With glmer(family=poisson),
### provided the models converge without Hessian warnings, the results are
### always very close to those from other methods. (The results are sound 
### even with boundary warnings. But the Hessian matrix warnings should 
### cause one to disregard the results. Fortunately, these warnings do not
### seem to happen that frequently, at least with the randomly generated
### data frames in this file.)


library(plyr) # for ddply
library(nnet) # for multinom
library(car) # for Anova
library(mlogit) # for mlogit.data
library(lme4) # for glmer
library(lmerTest)
library(multcomp)# for glht, mcp
library(emmeans) # for emm, emmeans


#set.seed(123)

p = 0.40  # probability of first response category



##
## 1. dichotomous response, one between-Ss. factor, two levels
##
df <- data.frame(
  S = factor(1:36, levels=1:36),
  X = factor(rep(c("a","b"), 18), levels=c("a","b")),
  Y = factor(sample(c("x","y"), 36, prob=c(p,1-p), replace=TRUE), levels=c("x","y"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##glm
m1.1 = glm(Y ~ X, data=df, family=binomial)
Anova(m1.1, type=3)

##multinom
m1.2 = multinom(Y ~ X, data=df)
Anova(m1.2, type=3)

##glmer
m1.3 = glmer(Y ~ X + (1|S), data=df, family=binomial)
Anova(m1.3, type=3)

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m1.4 = glm(Y ~ X*alt, data=df2, family=poisson)
a = Anova(m1.4, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

m1.5 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m1.5, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X




##
## 2. dichotomous response, one between-Ss. factor, three levels
##
df <- data.frame(
  S = factor(1:36, levels=1:36),
  X = factor(rep(c("a","b","c"), 12), levels=c("a","b","c")),
  Y = factor(sample(c("x","y"), 36, prob=c(p,1-p), replace=TRUE), levels=c("x","y"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##glm
m2.1 = glm(Y ~ X, data=df, family=binomial)
Anova(m2.1, type=3)
summary(glht(m2.1, emm(pairwise ~ X)), test=adjusted(type="holm"))

##multinom
m2.2 = multinom(Y ~ X, data=df)
Anova(m2.2, type=3)

df2 <- df[df$X == "a" | df$X == "b",] # a vs. b
df2$X = factor(df2$X)
contrasts(df2$X) <- "contr.sum"
m = multinom(Y ~ X, data=df2)
ab = Anova(m, type=3)

df2 <- df[df$X == "a" | df$X == "c",] # a vs. c
df2$X = factor(df2$X)
contrasts(df2$X) <- "contr.sum"
m = multinom(Y ~ X, data=df2)
ac = Anova(m, type=3)

df2 <- df[df$X == "b" | df$X == "c",] # b vs. c
df2$X = factor(df2$X)
contrasts(df2$X) <-"contr.sum"
m = multinom(Y ~ X, data=df2)
bc = Anova(m, type=3)

p.adjust(c(
  ab$`Pr(>Chisq)`, 
  ac$`Pr(>Chisq)`, 
  bc$`Pr(>Chisq)`), 
method="holm")

##glmer
m2.3 = glmer(Y ~ X + (1|S), data=df, family=binomial)
Anova(m2.3, type=3)
summary(glht(m2.3, emm(pairwise ~ X)), test=adjusted(type="holm"))

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m2.4 = glm(Y ~ X*alt, data=df2, family=poisson)
a = Anova(m2.4, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

m2.5 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m2.5, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

df3 <- df2[df2$X == "a" | df2$X == "b",] # a vs. b
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X == "a" | df2$X == "c",] # a vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ac = a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X == "b" | df2$X == "c",] # b vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
bc = a[grep(":alt", rownames(a)),]

p.adjust(c(
  ab$`Pr(>Chisq)`, 
  ac$`Pr(>Chisq)`, 
  bc$`Pr(>Chisq)`), 
method="holm")




##
## 3. dichotomous response, one within-Ss. factor, two levels
##
df <- data.frame(
  S = factor(rep(1:18, each=2), levels=1:18),
  X = factor(rep(c("a","b"), 18), levels=c("a","b")),
  Y = factor(sample(c("x","y"), 36, prob=c(p,1-p), replace=TRUE), levels=c("x","y"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##glmer
m3.1 = glmer(Y ~ X + (1|S), data=df, family=binomial)
Anova(m3.1, type=3)

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m3.2 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m3.2, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X




##
## 4. dichotomous response, one within-Ss. factor, three levels
##
df <- data.frame(
  S = factor(rep(1:12, each=3), levels=1:12),
  X = factor(rep(c("a","b","c"), 12), levels=c("a","b","c")),
  Y = factor(sample(c("x","y"), 36, prob=c(p,1-p), replace=TRUE), levels=c("x","y"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##glmer
m4.1 = glmer(Y ~ X + (1|S), data=df, family=binomial)
Anova(m4.1, type=3)
summary(glht(m4.1, emm(pairwise ~ X)), test=adjusted(type="holm"))

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m4.2 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m4.2, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

df3 <- df2[df2$X == "a" | df2$X == "b",] # a vs. b
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X == "a" | df2$X == "c",] # a vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ac = a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X == "b" | df2$X == "c",] # b vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
bc = a[grep(":alt", rownames(a)),]

p.adjust(c(
  ab$`Pr(>Chisq)`, 
  ac$`Pr(>Chisq)`, 
  bc$`Pr(>Chisq)`), 
method="holm")




##
## 5. dichotomous response, two between-Ss. factors, each two levels
##
df <- data.frame(
  S = factor(1:36, levels=1:36),
  X1 = factor(rep(c("a","b"), 9, each=2), levels=c("a","b")),
  X2 = factor(rep(c("a","b"), 18, each=1), levels=c("a","b")),
  Y = factor(sample(c("x","y"), 36, prob=c(p,1-p), replace=TRUE), levels=c("x","y"))
)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
summary(df)
ddply(df, ~ X1 + X2 + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##glm
m5.1 = glm(Y ~ X1*X2, data=df, family=binomial)
Anova(m5.1, type=3)
summary(glht(m5.1, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))

##multinom
m5.2 = multinom(Y ~ X1*X2, data=df)
Anova(m5.2, type=3)

df$X12 = with(df, interaction(X1,X2)) # make combined factor for contrasts

df2 <- df[df$X12 == "a.a" | df$X12 == "a.b",] # aa vs. ab
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_ab = Anova(m, type=3)

df2 <- df[df$X12 == "a.a" | df$X12 == "b.a",] # aa vs. ba
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_ba = Anova(m, type=3)

df2 <- df[df$X12 == "a.a" | df$X12 == "b.b",] # aa vs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_bb = Anova(m, type=3)

df2 <- df[df$X12 == "a.b" | df$X12 == "b.a",] # ab vs. ba
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ab_ba = Anova(m, type=3)

df2 <- df[df$X12 == "a.b" | df$X12 == "b.b",] # ab vs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ab_bb = Anova(m, type=3)

df2 <- df[df$X12 == "b.a" | df$X12 == "b.b",] # bavs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ba_bb = Anova(m, type=3)

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), 
method="holm")

##glmer
m5.3 = glmer(Y ~ X1*X2 + (1|S), data=df, family=binomial)
Anova(m5.3, type=3)
summary(glht(m5.3, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X1) <-"contr.sum"
contrasts(df2$X2) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m5.4 = glm(Y ~ X1*X2*alt, data=df2, family=poisson)
a = Anova(m5.4, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

m5.5 = glmer(Y ~ X1*X2*alt + (1|S), data=df2, family=poisson)
a = Anova(m5.5, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

df2$X12 = with(df2, interaction(X1,X2)) # make combined factor for contrasts

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "a.b",] # aa vs. ab
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ab= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.a",] # aa vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.b",] # aa vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.a",] # ab vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.b",] # ab vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "b.a" | df2$X12 == "b.b",] # bavs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ba_bb= a[grep(":alt", rownames(a)),]

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`,
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`),
method="holm")




##
## 6. dichotomous response, two within-Ss. factors, each two levels
##
df <- data.frame(
  S = factor(rep(1:9, each=4), levels=1:9),
  X1 = factor(rep(c("a","b"), 9, each=2), levels=c("a","b")),
  X2 = factor(rep(c("a","b"), 18, each=1), levels=c("a","b")),
  Y = factor(sample(c("x","y"), 36, prob=c(p,1-p), replace=TRUE), levels=c("x","y"))
)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
summary(df)
ddply(df, ~ X1 + X2 + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##glmer
m6.1 = glmer(Y ~ X1*X2 + (1|S), data=df, family=binomial)
Anova(m6.1, type=3)
summary(glht(m6.1, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X1) <-"contr.sum"
contrasts(df2$X2) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m6.2 = glmer(Y ~ X1*X2*alt + (1|S), data=df2, family=poisson)
a = Anova(m6.2, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

df2$X12 = with(df2, interaction(X1,X2)) # make combined factor for contrasts

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "a.b",] # aa vs. ab
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ab= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.a",] # aa vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.b",] # aa vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.a",] # ab vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.b",] # ab vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "b.a" | df2$X12 == "b.b",] # bavs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ba_bb= a[grep(":alt", rownames(a)),]

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), 
method="holm")




##
## 7. dichotomous response, mixed factorial design, each two levels
##
df <- data.frame(
  S = factor(rep(1:18, each=2), levels=1:18),
  X1 = factor(rep(c("a","b"), 9, each=2), levels=c("a","b")),
  X2 = factor(rep(c("a","b"), 18, each=1), levels=c("a","b")),
  Y = factor(sample(c("x","y"), 36, prob=c(p,1-p), replace=TRUE), levels=c("x","y"))
)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
summary(df)
ddply(df, ~ X1 + X2 + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##glmer
m7.1 = glmer(Y ~ X1*X2 + (1|S), data=df, family=binomial)
Anova(m7.1, type=3)
summary(glht(m7.1, emm(pairwise ~ X1*X2)), test=adjusted(type="holm"))

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X1) <-"contr.sum"
contrasts(df2$X2) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m7.2 = glmer(Y ~ X1*X2*alt + (1|S), data=df2, family=poisson)
a = Anova(m7.2, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

df2$X12 = with(df2, interaction(X1,X2)) # make combined factor for contrasts

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "a.b",] # aa vs. ab
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ab= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.a",] # aa vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.b",] # aa vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.a",] # ab vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.b",] # ab vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "b.a" | df2$X12 == "b.b",] # bavs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ba_bb= a[grep(":alt", rownames(a)),]

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), 
  method="holm")




## 
## 8. polytomous response, one between-Ss. factor, two levels
##
df <- data.frame(
  S = factor(1:36, levels=1:36),
  X = factor(rep(c("a","b"), 18), levels=c("a","b")),
  Y = factor(sample(c("x","y","z"), 36, prob=c(p/2,p/2,1-p), replace=TRUE), levels=c("x","y","z"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##multinom
m8.1 = multinom(Y ~ X, data=df)
Anova(m8.1, type=3)

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m8.2 = glm(Y ~ X*alt, data=df2, family=poisson)
a = Anova(m8.2, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

m8.3 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m8.3, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X




##
## 9. polytomous response, one between-Ss. factor, three levels
##
df <- data.frame(
  S = factor(1:36, levels=1:36),
  X = factor(rep(c("a","b","c"), 12), levels=c("a","b","c")),
  Y = factor(sample(c("x","y","z"), 36, prob=c(p/2,p/2,1-p), replace=TRUE), levels=c("x","y","z"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##multinom
m9.1 = multinom(Y ~ X, data=df)
Anova(m9.1, type=3)

df2 <- df[df$X == "a" | df$X == "b",] # a vs. b
df2$X = factor(df2$X)
contrasts(df2$X) <- "contr.sum"
m = multinom(Y ~ X, data=df2)
ab = Anova(m, type=3)

df2 <- df[df$X == "a" | df$X == "c",] # a vs. c
df2$X = factor(df2$X)
contrasts(df2$X) <- "contr.sum"
m = multinom(Y ~ X, data=df2)
ac = Anova(m, type=3)

df2 <- df[df$X == "b" | df$X == "c",] # b vs. c
df2$X = factor(df2$X)
contrasts(df2$X) <-"contr.sum"
m = multinom(Y ~ X, data=df2)
bc = Anova(m, type=3)

p.adjust(c(
  ab$`Pr(>Chisq)`, 
  ac$`Pr(>Chisq)`, 
  bc$`Pr(>Chisq)`), 
  method="holm")

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m9.2 = glm(Y ~ X*alt, data=df2, family=poisson)
a = Anova(m9.2, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

m9.3 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m9.3, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

df3 <- df2[df2$X == "a" | df2$X == "b",] # a vs. b
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X == "a" | df2$X == "c",] # a vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ac = a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X == "b" | df2$X == "c",] # b vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
bc = a[grep(":alt", rownames(a)),]

p.adjust(c(
  ab$`Pr(>Chisq)`, 
  ac$`Pr(>Chisq)`, 
  bc$`Pr(>Chisq)`), 
  method="holm")




##
## 10. polytomous response, one within-Ss. factor, two levels
##
df <- data.frame(
  S = factor(rep(1:18, each=2), levels=1:18),
  X = factor(rep(c("a","b"), 18), levels=c("a","b")),
  Y = factor(sample(c("x","y","z"), 36, prob=c(p/2,p/2,1-p), replace=TRUE), levels=c("x","y","z"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m10.1 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m10.1, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

m10.2 = glmer.nb(Y ~ X*alt + (1|S), data=df2, control=glmerControl(optCtrl=list(maxfun=1e5))) # try negative binomial
a = Anova(m10.2, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.




##
## 11. polytomous response, one within-Ss. factor, three levels
##
df <- data.frame(
  S = factor(rep(1:12, each=3), levels=1:12),
  X = factor(rep(c("a","b","c"), 12), levels=c("a","b","c")),
  Y = factor(sample(c("x","y","z"), 36, prob=c(p/2,p/2,1-p), replace=TRUE), levels=c("x","y","z"))
)
contrasts(df$X) <- "contr.sum"
summary(df)
ddply(df, ~ X + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m11.1 = glmer(Y ~ X*alt + (1|S), data=df2, family=poisson)
a = Anova(m11.1, type=3)
a[grep(":alt", rownames(a)),] # X:alt is X

m11.2 = glmer.nb(Y ~ X*alt + (1|S), data=df2, control=glmerControl(optCtrl=list(maxfun=1e5))) # try negative binomial
a = Anova(m11.2, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

df3 <- df2[df2$X == "a" | df2$X == "b",] # a vs. b
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab = a[grep(":alt", rownames(a)),]

df3 <- df2[df2$X == "a" | df2$X == "c",] # a vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ac = a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X == "b" | df2$X == "c",] # b vs. c
df3$X = factor(df3$X)
contrasts(df3$X) <-"contr.sum"
m = glmer(Y ~ X*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
bc = a[grep(":alt", rownames(a)),]

p.adjust(c(
  ab$`Pr(>Chisq)`, 
  ac$`Pr(>Chisq)`, 
  bc$`Pr(>Chisq)`), 
  method="holm")




##
## 12. polytomous response, two between-Ss. factors, each two levels
##
df <- data.frame(
  S = factor(1:36, levels=1:36),
  X1 = factor(rep(c("a","b"), 9, each=2), levels=c("a","b")),
  X2 = factor(rep(c("a","b"), 18, each=1), levels=c("a","b")),
  Y = factor(sample(c("x","y","z"), 36, prob=c(p/2,p/2,1-p), replace=TRUE), levels=c("x","y","z"))
)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
summary(df)
ddply(df, ~ X1 + X2 + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##multinom
m12.1 = multinom(Y ~ X1*X2, data=df)
Anova(m12.1, type=3)

df$X12 = with(df, interaction(X1,X2)) # make combined factor for contrasts

df2 <- df[df$X12 == "a.a" | df$X12 == "a.b",] # aa vs. ab
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_ab = Anova(m, type=3)

df2 <- df[df$X12 == "a.a" | df$X12 == "b.a",] # aa vs. ba
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_ba = Anova(m, type=3)

df2 <- df[df$X12 == "a.a" | df$X12 == "b.b",] # aa vs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
aa_bb = Anova(m, type=3)

df2 <- df[df$X12 == "a.b" | df$X12 == "b.a",] # ab vs. ba
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ab_ba = Anova(m, type=3)

df2 <- df[df$X12 == "a.b" | df$X12 == "b.b",] # ab vs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ab_bb = Anova(m, type=3)

df2 <- df[df$X12 == "b.a" | df$X12 == "b.b",] # bavs. bb
df2$X12 = factor(df2$X12)
contrasts(df2$X12) <- "contr.sum"
m = multinom(Y ~ X12, data=df2)
ba_bb = Anova(m, type=3)

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), 
  method="holm")

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X1) <-"contr.sum"
contrasts(df2$X2) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m12.2 = glm(Y ~ X1*X2*alt, data=df2, family=poisson)
a = Anova(m12.2, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

m12.3 = glmer(Y ~ X1*X2*alt + (1|S), data=df2, family=poisson)
a = Anova(m12.3, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

m12.4 = glmer.nb(Y ~ X1*X2*alt + (1|S), data=df2, control=glmerControl(optCtrl=list(maxfun=1e5))) # try negative binomial
a = Anova(m12.4, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

df2$X12 = with(df2, interaction(X1,X2)) # make combined factor for contrasts

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "a.b",] # aa vs. ab
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ab= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.a",] # aa vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.b",] # aa vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.a",] # ab vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.b",] # ab vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "b.a" | df2$X12 == "b.b",] # bavs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ba_bb= a[grep(":alt", rownames(a)),]

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`,
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`),
  method="holm")




##
## 13. polytomous response, two within-Ss. factors, each two levels
##
df <- data.frame(
  S = factor(rep(1:9, each=4), levels=1:9),
  X1 = factor(rep(c("a","b"), 9, each=2), levels=c("a","b")),
  X2 = factor(rep(c("a","b"), 18, each=1), levels=c("a","b")),
  Y = factor(sample(c("x","y","z"), 36, prob=c(p/2,p/2,1-p), replace=TRUE), levels=c("x","y","z"))
)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
summary(df)
ddply(df, ~ X1 + X2 + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X1) <-"contr.sum"
contrasts(df2$X2) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m13.1 = glmer(Y ~ X1*X2*alt + (1|S), data=df2, family=poisson)
a = Anova(m13.1, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

m13.2 = glmer.nb(Y ~ X1*X2*alt + (1|S), data=df2, control=glmerControl(optCtrl=list(maxfun=1e5))) # try negative binomial
a = Anova(m13.2, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

df2$X12 = with(df2, interaction(X1,X2)) # make combined factor for contrasts

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "a.b",] # aa vs. ab
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ab= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.a",] # aa vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.b",] # aa vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.a",] # ab vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.b",] # ab vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "b.a" | df2$X12 == "b.b",] # bavs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ba_bb= a[grep(":alt", rownames(a)),]

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), 
  method="holm")




##
## 14. polytomous response, mixed factorial design, each two levels
##
df <- data.frame(
  S = factor(rep(1:18, each=2), levels=1:18),
  X1 = factor(rep(c("a","b"), 9, each=2), levels=c("a","b")),
  X2 = factor(rep(c("a","b"), 18, each=1), levels=c("a","b")),
  Y = factor(sample(c("x","y","z"), 36, prob=c(p/2,p/2,1-p), replace=TRUE), levels=c("x","y","z"))
)
contrasts(df$X1) <- "contr.sum"
contrasts(df$X2) <- "contr.sum"
summary(df)
ddply(df, ~ X1 + X2 + Y, function(data) c("Nrows"=nrow(data)))
View(df)

##m-P
df2 <- as.data.frame(mlogit.data(df, choice="Y", shape="wide", id.var="S"))
df2$S = factor(df2$S)
contrasts(df2$X1) <-"contr.sum"
contrasts(df2$X2) <-"contr.sum"
contrasts(df2$alt) <-"contr.sum"

m14.1 = glmer(Y ~ X1*X2*alt + (1|S), data=df2, family=poisson)
a = Anova(m14.1, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

m14.2 = glmer.nb(Y ~ X1*X2*alt + (1|S), data=df2, control=glmerControl(optCtrl=list(maxfun=1e5))) # try negative binomial
a = Anova(m14.2, type=3)
a[grep(":alt", rownames(a)),] # X1:alt is X1, etc.

df2$X12 = with(df2, interaction(X1,X2)) # make combined factor for contrasts

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "a.b",] # aa vs. ab
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ab= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.a",] # aa vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.a" | df2$X12 == "b.b",] # aa vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
aa_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.a",] # ab vs. ba
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_ba= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "a.b" | df2$X12 == "b.b",] # ab vs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ab_bb= a[grep(":alt", rownames(a)),]

df3 <-df2[df2$X12 == "b.a" | df2$X12 == "b.b",] # bavs. bb
df3$X12 = factor(df3$X12)
contrasts(df3$X12) <-"contr.sum"
m = glmer(Y ~ X12*alt + (1|S), data=df3, family=poisson)
a = Anova(m, type=3)
ba_bb= a[grep(":alt", rownames(a)),]

p.adjust(c(
  aa_ab$`Pr(>Chisq)`, 
  aa_ba$`Pr(>Chisq)`, 
  aa_bb$`Pr(>Chisq)`, 
  ab_ba$`Pr(>Chisq)`, 
  ab_bb$`Pr(>Chisq)`, 
  ba_bb$`Pr(>Chisq)`), 
  method="holm")



